asympteq2by2 <- function(crit,p1,q1,np,nq) {


#  Returns the power of Pearson's statistic on 2 by 2 tables
#  caclulated by the asymptotic equivalent.
#
#  INPUT DATA
#        crit = critical value
#          p1 = probability that a controls falls in category 1.
#          q1 = probability that a case falls in category 1.
#          np = number of controls
#          nq = number of cases
#
#  OUTPUT DATA is the power calculated by the asymptotic equivalent.
#
#  CODES NEDEED: doublenorm
#

  if (p1==q1)  power <-  1 - pchisq(crit,1) # Although it is not really power, because p1 = q1 holds under the null hypothesis.
  else {

   n <- np+nq
   pp <- np/n
   qq <- nq/n

   p2 <- 1-p1
   q2 <- 1-q1


   aux1 <- ((p1-q1)*qq)/(2*(p1*pp+q1*qq)) + 1
   aux2 <- ((p2-q2)*qq)/(2*(p2*pp+q2*qq)) + 1
   bux1 <- -((p1-q1)*pp)/(2*(p1*pp+q1*qq)) + 1
   bux2 <- -((p2-q2)*pp)/(2*(p2*pp+q2*qq)) + 1

   B11 <- (aux1*aux1*pp*q1*q2 + bux1*bux1*qq*p1*p2) / (p1*pp+q1*qq)
   B22 <- (aux2*aux2*pp*q1*q2 + bux2*bux2*qq*p1*p2) / (p2*pp+q2*qq)
   B12 <- -(aux1*aux2*pp*q1*q2 + bux1*bux2*qq*p1*p2) / sqrt( (p1*pp+q1*qq)*(p2*pp+q2*qq) )
   B21 <- B12


# Eigenvalue calculation

   b <- -(B11+B22)
   c <- B11*B22-B12*B21

   eig1 <- ( -b + sqrt(b*b-4*c) ) * 0.5
   eig2 <- ( -b - sqrt(b*b-4*c) ) * 0.5

# Eigenvector calculation
  
   v1 <- -B12
   v2 <- B11 - eig1
   leng <- sqrt( v1*v1 + v2*v2 )
   v11 = v1 / leng
   v12 = v2 / leng

   v1 <- -B12
   v2 <- B11 - eig2
   leng <- sqrt( v1*v1 + v2*v2 )
   v21 = v1 / leng
   v22 = v2 / leng

# Expected value calculation

   expect1 <- ( v11*(q1-p1)/sqrt(pp*p1+qq*q1) + v12*(q2-p2)/sqrt(pp*p2+qq*q2) ) * sqrt(n*pp*qq)
   expect2 <- ( v21*(q1-p1)/sqrt(pp*p1+qq*q1) + v22*(q2-p2)/sqrt(pp*p2+qq*q2) ) * sqrt(n*pp*qq)

   K <- 1 / eig1
   L <- 1 / eig2

   lam1 <- K*expect1*expect1
   lam2 <- L*expect2*expect2

   powac <- doublenorm(crit,K,L,lam1,lam2,10000)
   power <- 1 - powac[1]

  }

  power

}




