costeff2by2 <- function(crit,p1,q1,np,nq) {

#  Returns the power of Pearson's statistic on 2 by 2 tables
#  caclulated by the cost-effective method.
#
#  INPUT DATA
#        crit = critical value
#          p1 = probability that a controls falls in category 1.
#          q1 = probability that a case falls in category 1.
#          np = number of controls
#          nq = number of cases
#
#  OUTPUT DATA is the power calculated by the cost-effective method.
#
#  CODES NEDEED: -
#

   n <- np+nq
   pp <- np/n
   qq <- nq/n


   p2 <- 1-p1
   q2 <- 1-q1

   aux1 <- ((p1-q1)*qq)/(2*(p1*pp+q1*qq)) + 1
   aux2 <- ((p2-q2)*qq)/(2*(p2*pp+q2*qq)) + 1
   bux1 <- -((p1-q1)*pp)/(2*(p1*pp+q1*qq)) + 1
   bux2 <- -((p2-q2)*pp)/(2*(p2*pp+q2*qq)) + 1

   B11 <- (aux1*aux1*pp*q1*q2 + bux1*bux1*qq*p1*p2) / (p1*pp+q1*qq)
   B22 <- (aux2*aux2*pp*q1*q2 + bux2*bux2*qq*p1*p2) / (p2*pp+q2*qq)
   B12 <- -(aux1*aux2*pp*q1*q2 + bux1*bux2*qq*p1*p2) / sqrt( (p1*pp+q1*qq)*(p2*pp+q2*qq) )
   B21 <- B12


# Eigenvalue calculation

   d <- B11 - B22
   eig1 <- ( B11 + B22 + sqrt(d*d+4*B12*B21) ) * 0.5


# Power calculation
  
   coeffo <- 1 / eig1
   noncold <- ((p1-q1)*(p1-q1)*n*pp*qq) / ((p1*pp+q1*qq)*(p2*pp+q2*qq))
   nonco <- coeffo * noncold
   power <- 1 - pchisq(coeffo*crit, 1, ncp=nonco)

   power

}




