doublenorm <- function(t,K,L,lam1,lam2,N) {

#  Returns the probability that X/K + Y/L < t,
#  where X and Y are independent chi-squared random 
#  variables with d.f. 1 and non-centrality parameters
#  lam1 and lam2, respectively.
#  N is the number of partitioning points used for 
#  numerical integration.
#
#  OUTPUT is an array of 2 real numbers. The first one 
#  is the probability that X/K + Y/L < t, the second one is
#  an upper bound for the computation error.
#          
#  CODES NEDEED: -
#

     a <- K * t
     b <- L * t

     dist <- a/N

     x <- seq(0, a, by=dist)

     aux <- pchisq(x, df=1, ncp=lam1)
     densi <- rep(0,N)
     for (i in 1:N) densi[i] = aux[i+1] - aux[i]  

     y <- (t-x/K)*L

     prol <- pchisq(y[2:(N+1)], df=1, ncp=lam2)
     lobound <- sum(densi * prol)

     prou <- pchisq(y[1:N], df=1, ncp=lam2)
     upbound <- sum(densi * prou)

     value <- c( (lobound+upbound)/2, (upbound-lobound) )

     value

}    